searchNodes=[{"doc":"DataSchemas are declarative specifications of how to create structs from some kind of data source. For example you can define a schema that knows how to turn an elixir map into a struct, casting all of the values as it goes. Alternatively you can set up a schema to ingest XML data and create structs from the values inside the XML. Below is an example of a simple schema: defmodule Blog do import DataSchema , only : [ data_schema : 1 ] data_schema ( [ field : { :name , &quot;name&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end This says we will create a struct with a :name key and will get the value for that key from under the &quot;name&quot; key in the source data. That value will be passed to to_string/1 and the result of that function will end up as the value under :name in the resulting struct. In general this is the format for a field: field { :content , &quot;text&quot; , &amp; cast_string / 1 } # ^ ^ ^ ^ # field type | | | # struct key name | | # path to data in the source | # casting function Depending on your input data type the path pointing to a value in it may need to be interpreted differently. For our example of a map input type, the &quot;path&quot; is really just a key on that input map. But there is still flexibility in how we use that key to access the value; we could use Map.get/2 or Map.fetch/2 for example. Additionally, for different input data types what the path looks like and what it means for how you access data can be different. Let's say your input data type was XML your path could be &quot;.//MyNode&quot;, ie could be an xpath. In which case what you do with that xpath is going to be different from what you would do with a map key. DataSchema allows for different schemas to handle different input types AND allows for the same input type to be handled differently in different schemas. Finally when creating the struct we can choose to stop as soon as we find an error or to simply put whatever is returned from a casting function into the struct we are making. The latter approach encourages people to raise exceptions from their casting functions to halt the creation of the struct. Field Types There are 5 kinds of struct fields we can have: field - The value will be a casted value from the source data. list_of - The value will be a list of casted values created from the source data. has_one - The value will be created from a nested data schema (so will be a struct) has_many - The value will be created by casting a list of values into a data schema. (You end up with a list of structs defined by the provided schema). Similar to has_many in ecto aggregate - The value will a casted value formed from multiple bits of data in the source. Examples See the guides for more in depth examples but below you can see how we create a schema that will take a map of data and create a struct out of it. Given the following schema: defmodule Sandwich do require DataSchema DataSchema . data_schema ( [ field : { :type , &quot;the_type&quot; , &amp; { :ok , String . upcase ( &amp;1 ) } } , list_of : { :fillings , &quot;the_fillings&quot; , &amp; ( { :ok , String . downcase ( &amp;1 [ &quot;name&quot; ] ) } ) } ] ) end input_data = %{ &quot;the_type&quot; =&gt; &quot;fake steak&quot; , &quot;the_fillings&quot; =&gt; [ %{ &quot;name&quot; =&gt; &quot;fake stake&quot; , &quot;good?&quot; =&gt; true } , %{ &quot;name&quot; =&gt; &quot;SAUCE&quot; } , %{ &quot;name&quot; =&gt; &quot;sweetcorn&quot; } , ] } DataSchema . to_struct ( input_data , Sandwich ) # outputs the following: % Sandwich { type : &quot;FAKE STEAK&quot; , fillings : [ &quot;fake stake&quot; , &quot;sauce&quot; , &quot;sweetcorn&quot; ] , }","ref":"DataSchema.html","title":"DataSchema","type":"module"},{"doc":"A macro that creates a data schema. By default all struct fields are required but you can specify that a field be optional by passing the correct option in. See the Options section below for more. Field Types There are 5 kinds of struct fields we can have: field - The value will be a casted value from the source data. list_of - The value will be a list of casted values created from the source data. has_one - The value will be created from a nested data schema (so will be a struct) has_many - The value will be created by casting a list of values into a data schema. (You end up with a list of structs defined by the provided schema). Similar to has_many in ecto aggregate - The value will a casted value formed from multiple bits of data in the source. Options Available options are: :optional? - specifies whether or not the field in the struct should be included in the @enforce_keys for the struct. By default all fields are required but you can mark them as optional by setting this to true . This will also be checked when creating a struct with DataSchema.to_struct/2 returning an error if the required field is null. For example: defmodule Sandwich do require DataSchema DataSchema . data_schema ( [ field : { :type , &quot;the_type&quot; , &amp; { :ok , String . upcase ( &amp;1 ) } , optional? : true } , ] ) end Examples See the guides for more in depth examples but below you can see how we create a schema that will take a map of data and create a struct out of it. Given the following schema: defmodule Sandwich do require DataSchema DataSchema . data_schema ( [ field : { :type , &quot;the_type&quot; , &amp; { :ok , String . upcase . ( &amp;1 ) } } , list_of : { :fillings , &quot;the_fillings&quot; , &amp; ( { :ok , String . downcase ( &amp;1 [ &quot;name&quot; ] ) } ) } ] ) end input_data = %{ &quot;the_type&quot; =&gt; &quot;fake steak&quot; , &quot;the_fillings&quot; =&gt; [ %{ &quot;name&quot; =&gt; &quot;fake stake&quot; , &quot;good?&quot; =&gt; true } , %{ &quot;name&quot; =&gt; &quot;SAUCE&quot; } , %{ &quot;name&quot; =&gt; &quot;sweetcorn&quot; } , ] } DataSchema . to_struct ( input_data , Sandwich ) # outputs the following: % Sandwich { type : &quot;FAKE STEAK&quot; , fillings : [ &quot;fake stake&quot; , &quot;sauce&quot; , &quot;sweetcorn&quot; ] , }","ref":"DataSchema.html#data_schema/1","title":"DataSchema.data_schema/1","type":"macro"},{"doc":"Accepts an data schema module and some source data and attempts to create the struct defined in the schema from the source data recursively. We essentially visit each field in the schema and extract the data the field points to from the sauce data, passing it to the field's casting function before setting the result of that as the value on the struct. This function takes a simple approach to creating the struct - whatever you return from a casting function will be set as the value of the struct field. You should raise if you want casting to fail. Examples data = %{ &quot;spice&quot; =&gt; &quot;enables space travel&quot; } defmodule Foo do require DataSchema DataSchema . data_schema ( field : { :a_rocket , &quot;spice&quot; , &amp; ( { :ok , &amp;1 } ) } ) end DataSchema . to_struct ( data , Foo ) # =&gt; Outputs the following: % Foo { a_rocket : &quot;enables space travel&quot; }","ref":"DataSchema.html#to_struct/2","title":"DataSchema.to_struct/2","type":"function"},{"doc":"","ref":"DataSchema.html#to_struct/3","title":"DataSchema.to_struct/3","type":"function"},{"doc":"","ref":"DataSchema.html#to_struct/4","title":"DataSchema.to_struct/4","type":"function"},{"doc":"Creates a struct or map from the provided arguments. This function can be used to define runtime schemas for the most dynamic of cases. This means you don't have to define a schema at compile time using the DataShema.data_schema/1 macro. Examples Creating a struct: defmodule Run do defstruct [ :time ] end input = %{ &quot;time&quot; =&gt; &quot;10:00&quot; } fields = [ field : { :time , &quot;time&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] DataSchema . to_struct ( input , Run , fields , DataSchema.MapAccessor ) { :ok , % Run { time : &quot;10:00&quot; } } Creating a map: input = %{ &quot;time&quot; =&gt; &quot;10:00&quot; } fields = [ field : { :time , &quot;time&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] DataSchema . to_struct ( input , %{ } , fields , DataSchema.MapAccessor ) { :ok , %{ time : &quot;10:00&quot; } }","ref":"DataSchema.html#to_struct/5","title":"DataSchema.to_struct/5","type":"function"},{"doc":"The casting function can either be a 1 arity function or a module that has a cast/1 function implemented. This behaviour is to help ensure that cast/1 function looks the way DataSchema expects.","ref":"DataSchema.CastBehaviour.html","title":"DataSchema.CastBehaviour","type":"behaviour"},{"doc":"A doc should return an okay tuple with the casted value or an error tuple. For errors you can optionally provide a message that will be returned if DataSchema.to_struct/2 fails.","ref":"DataSchema.CastBehaviour.html#c:cast/1","title":"DataSchema.CastBehaviour.cast/1","type":"callback"},{"doc":"When we create a struct we either return the struct we were creating or we return this error. The error/errors that happened during struct creation are collected into this struct","ref":"DataSchema.Errors.html","title":"DataSchema.Errors","type":"module"},{"doc":"Adds an error to the given errors struct. The error is prepended to the list of current errors.","ref":"DataSchema.Errors.html#add_error/2","title":"DataSchema.Errors.add_error/2","type":"function"},{"doc":"An error is the struct key that caused the error and either an error message or a DataSchema.Errors struct in the case of nested error.","ref":"DataSchema.Errors.html#t:t/0","title":"DataSchema.Errors.t/0","type":"type"},{"doc":"Defines a way to use elixir maps as an input data source. This will Map.get/2 the fields from the source data, meaning it will return nil in the case of the key being missing.","ref":"DataSchema.MapAccessor.html","title":"DataSchema.MapAccessor","type":"module"},{"doc":"Accesses the source data using Map.get/2 when a :field type is encountered during struct creation. If the incoming data is not a map we will function clause error.","ref":"DataSchema.MapAccessor.html#field/2","title":"DataSchema.MapAccessor.field/2","type":"function"},{"doc":"Accesses the source data using Map.get/2 when a :has_many type is encountered during struct creation.","ref":"DataSchema.MapAccessor.html#has_many/2","title":"DataSchema.MapAccessor.has_many/2","type":"function"},{"doc":"Accesses the source data using Map.get/2 when a :has_one type is encountered during struct creation.","ref":"DataSchema.MapAccessor.html#has_one/2","title":"DataSchema.MapAccessor.has_one/2","type":"function"},{"doc":"Accesses the source data using Map.get/2 when a :list_of type is encountered during struct creation.","ref":"DataSchema.MapAccessor.html#list_of/2","title":"DataSchema.MapAccessor.list_of/2","type":"function"},{"doc":"Defines how DataSchema.to_struct/2 should access data for each given field type. When we create a struct defined by the schema we will visit each struct field in turn and attempt to extract values from the source data. You tell the schema how to extract that data by providing a module that implements this behaviour. Modules that implement this behaviour are implementing how to access data in the source for each kind of field. Examples To define a schema that consumes a map to create a struct you would first create a module that implements this behaviour like so: defmodule DataSchema.MapAccessor do @behaviour DataSchema.DataAccessBehaviour @impl true def field ( data = %{ } , field ) do Map . get ( data , field ) end @impl true def list_of ( data = %{ } , field ) do Map . get ( data , field ) end @impl true def has_one ( data = %{ } , field ) do Map . get ( data , field ) end @impl true def has_many ( data = %{ } , field ) do Map . get ( data , field ) end end Then you would pass it into the schema definition: defmodule BlogPost do import DataSchema , only : [ data_schema : 2 ] data_schema ( [ field : { :content , &quot;content&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] , DataSchema.MapAccessor ) end When we call DataSchema.to_struct/2 the functions in DataSchema.MapAccessor will be used to access the data from the source. DataSchema ships with the above example (see DataSchema.MapAccessor ) and you can use DataSchema.data_schema/1 to leverage it automatically.","ref":"DataSchema.DataAccessBehaviour.html","title":"DataSchema.DataAccessBehaviour","type":"behaviour"},{"doc":"The function that will be called when a :field field is encountered in the schema when we are turning some input data into a struct.","ref":"DataSchema.DataAccessBehaviour.html#c:field/2","title":"DataSchema.DataAccessBehaviour.field/2","type":"callback"},{"doc":"The function that will be called when a :has_one field is encountered in the schema when we are turning some input data into a struct.","ref":"DataSchema.DataAccessBehaviour.html#c:has_many/2","title":"DataSchema.DataAccessBehaviour.has_many/2","type":"callback"},{"doc":"The function that will be called when a :has_one field is encountered in the schema when we are turning some input data into a struct.","ref":"DataSchema.DataAccessBehaviour.html#c:has_one/2","title":"DataSchema.DataAccessBehaviour.has_one/2","type":"callback"},{"doc":"The function that will be called when a :list_of field is encountered in the schema when we are turning some input data into a struct.","ref":"DataSchema.DataAccessBehaviour.html#c:list_of/2","title":"DataSchema.DataAccessBehaviour.list_of/2","type":"callback"},{"doc":"An error for when a schema is specified incorrectly. Usually this means there has been a syntax error when defining the schema.","ref":"DataSchema.InvalidSchemaError.html","title":"DataSchema.InvalidSchemaError","type":"exception"},{"doc":"Data schemas are declarative descriptions of how to create a struct from some input data. You can set up different schemas to handle different kinds of input data. By default we assume the incoming data is a map, but you can configure schemas to work with any arbitrary data input including XML and json. Data Schemas really shine when working with API responses - converting the response into trusted internal data easily and efficiently.","ref":"getting-started.html","title":"What Are Data Schemas?","type":"extras"},{"doc":"Let's think of creating a struct as taking some source data and turning it into the desired struct. To do this we need to know at least three things: The keys of the desired struct The types of the values for each of the keys Where / how to get the data for each value from the source data. Turning the source data into the correct type defined by the schema will often require casting, so to cater for that the type definitions are casting functions. Let's look at a simple field example field { :content , &quot;text&quot; , &amp; cast_string / 1 } # ^ ^ ^ # struct field | | # path to data in the source | # casting function This says in the source data there will be a field called &quot;text&quot; . When creating a struct we should get the data under that field and pass it too cast_string/1 . The result of that function will be put in the resultant struct under the key :content . There are 5 kinds of struct fields we could want: field - The value will be a casted value from the source data. list_of - The value will be a list of casted values created from the source data. has_one - The value will be created from a nested data schema (so will be a struct) has_many - The value will be created by casting a list of values into a data schema. (You end up with a list of structs defined by the provided schema). Similar to has_many in ecto aggregate - The value will a casted value formed from multiple bits of data in the source. To see this better let's look at a very simple example. Assume our input data looks like this: source_data = %{ &quot;content&quot; =&gt; &quot;This is a blog post&quot; , &quot;comments&quot; =&gt; [ %{ &quot;text&quot; =&gt; &quot;This is a comment&quot; } , %{ &quot;text&quot; =&gt; &quot;This is another comment&quot; } ] , &quot;draft&quot; =&gt; %{ &quot;content&quot; =&gt; &quot;This is a draft blog post&quot; } , &quot;date&quot; =&gt; &quot;2021-11-11&quot; , &quot;time&quot; =&gt; &quot;14:00:00&quot; , &quot;metadata&quot; =&gt; %{ &quot;rating&quot; =&gt; 0 } } And now let's assume the struct we wish to make is this one: % BlogPost { content : &quot;This is a blog post&quot; , comments : [ % Comment { text : &quot;This is a comment&quot; } , % Comment { text : &quot;This is another comment&quot; } ] , draft : % DraftPost { content : &quot;This is a draft blog post&quot; } , post_datetime : ~N[2020-11-11 14:00:00] } We can describe the following schemas to enable this: defmodule DraftPost do import DataSchema , only : [ data_schema : 1 ] data_schema ( [ field : { :content , &quot;content&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end defmodule Comment do import DataSchema , only : [ data_schema : 1 ] data_schema ( [ field : { :text , &quot;text&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end defmodule BlogPost do import DataSchema , only : [ data_schema : 1 ] @date_time_fields [ field : { :date , &quot;date&quot; , &amp; Date . from_iso8601 / 1 } , field : { :time , &quot;time&quot; , &amp; Time . from_iso8601 / 1 } ] data_schema ( [ field : { :content , &quot;content&quot; , &amp; { :ok , to_string ( &amp;1 ) } } , has_many : { :comments , &quot;comments&quot; , Comment } , has_one : { :draft , &quot;draft&quot; , DraftPost } , aggregate : { :post_datetime , @date_time_fields , &amp; NaiveDateTime . new ( &amp;1 . date , &amp;1 . time ) } , ] ) end Then to transform some input data into the desired struct we can call DataSchema.to_struct/2 which works recursively to transform the input data into the struct defined by the schema. source_data = %{ &quot;content&quot; =&gt; &quot;This is a blog post&quot; , &quot;comments&quot; =&gt; [ %{ &quot;text&quot; =&gt; &quot;This is a comment&quot; } , %{ &quot;text&quot; =&gt; &quot;This is another comment&quot; } ] , &quot;draft&quot; =&gt; %{ &quot;content&quot; =&gt; &quot;This is a draft blog post&quot; } , &quot;date&quot; =&gt; &quot;2021-11-11&quot; , &quot;time&quot; =&gt; &quot;14:00:00&quot; , &quot;metadata&quot; =&gt; %{ &quot;rating&quot; =&gt; 0 } } DataSchema . to_struct ( source_data , BlogPost ) # This will output the following: { :ok , % BlogPost { content : &quot;This is a blog post&quot; , comments : [ % Comment { text : &quot;This is a comment&quot; } , % Comment { text : &quot;This is another comment&quot; } ] draft : % DraftPost { content : &quot;This is a draft blog post&quot; } } post_datetime : ~N[2020-11-11 14:00:00] } }","ref":"getting-started.html#creating-a-simple-schema","title":"What Are Data Schemas? - Creating A Simple Schema","type":"extras"},{"doc":"As we mentioned before we want to be able to handle multiple different kinds of source data in our schemas. For each type of source data we want to be able to specify how you access the data for each field type. We do that by providing a &quot;data accessor&quot; (a module that implements the DataSchema.DataAccessBehaviour ) when we create the schema. We do this by providing a @data_accessor on the schema. By default if you do not provide this module attribute we use DataSchema.MapAccessor . That means the above example is equivalent to doing the following: defmodule DraftPost do import DataSchema , only : [ data_schema : 1 ] @data_accessor DataSchema.MapAccessor data_schema ( [ field : { :content , &quot;content&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end defmodule Comment do import DataSchema , only : [ data_schema : 1 ] @data_accessor DataSchema.MapAccessor data_schema ( [ field : { :text , &quot;text&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end defmodule BlogPost do import DataSchema , only : [ data_schema : 1 ] @data_accessor DataSchema.MapAccessor @date_time_fields [ field : { :date , &quot;date&quot; , &amp; Date . from_iso8601 / 1 } , field : { :time , &quot;time&quot; , &amp; Time . from_iso8601 / 1 } ] data_schema ( [ field : { :content , &quot;content&quot; , &amp; { :ok , to_string ( &amp;1 ) } } , has_many : { :comments , &quot;comments&quot; , Comment } , has_one : { :draft , &quot;draft&quot; , DraftPost } , aggregate : { :post_datetime , @date_time_fields , &amp; NaiveDateTime . new ( &amp;1 . date , &amp;1 . time ) } , ] ) end When creating the struct DataSchema will call the relevant function for the field we are creating, passing it the source data and the path to the value(s) in the source. Our DataSchema.MapAccessor looks like this: defmodule DataSchema.MapAccessor do @behaviour DataSchema.DataAccessBehaviour @impl true def field ( data = %{ } , field ) do Map . get ( data , field ) end @impl true def list_of ( data = %{ } , field ) do Map . get ( data , field ) end @impl true def has_one ( data = %{ } , field ) do Map . get ( data , field ) end @impl true def has_many ( data = %{ } , field ) do Map . get ( data , field ) end end To save repeating @data_accessor DataSchema.MapAccessor on all of your schemas you could use a __using__ macro like so: defmodule MapSchema do defmacro __using__ ( _ ) do quote do import DataSchema , only : [ data_schema : 1 ] @data_accessor DataSchema.MapAccessor end end end Then use it like so: defmodule DraftPost do use MapSchema data_schema ( [ field : { :content , &quot;content&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end This means should we want to change how we access data (say we wanted to use Map.fetch! instead of Map.get ) we would only need to change the accessor used in one place - inside the __using__ macro. It also gives you a handy place to provide other functions for the structs that get created, perhaps implementing a default Inspect protocol implementation for example: defmodule MapSchema do defmacro __using__ ( opts ) do skip_inspect_impl = Keyword . get ( opts , :skip_inspect_impl , false ) quote bind_quoted : [ skip_inspect_impl : skip_inspect_impl ] do import DataSchema , only : [ data_schema : 1 ] @data_accessor DataSchema.MapAccessor unless skip_inspect_impl do defimpl Inspect do def inspect ( struct , _opts ) do &quot;&lt;&quot; &lt;&gt; &quot; \#{ struct . __struct__ } &quot; &lt;&gt; &quot;&gt;&quot; end end end end end end This could help ensure you never log sensitive fields by requiring you to explicitly implement an inspect protocol for a struct in order to see the fields in it.","ref":"getting-started.html#different-source-data-types","title":"What Are Data Schemas? - Different Source Data Types","type":"extras"},{"doc":"XML Schemas Let's imagine that we have some XML that we wish to turn into a struct. What would it require to enable that? First a new Xpath data accessor: defmodule XpathAccessor do @behaviour DataSchema.DataAccessBehaviour import SweetXml , only : [ sigil_x : 2 ] @impl true def field ( data , path ) do SweetXml . xpath ( data , ~x&quot; \#{ path } &quot;s ) end @impl true def list_of ( data , path ) do SweetXml . xpath ( data , ~x&quot; \#{ path } &quot;l ) end @impl true def has_one ( data , path ) do SweetXml . xpath ( data , ~x&quot; \#{ path } &quot; ) end @impl true def has_many ( data , path ) do SweetXml . xpath ( data , ~x&quot; \#{ path } &quot;l ) end end As we can see our accessor uses the library Sweet XML to access the XML. That means if we wanted to change the library later we would only need to alter this one module for all of our schemas to benefit from the change. Our source data looks like this: source_data = &quot;&quot;&quot; &lt;Blog date=&quot;2021-11-11&quot; time=&quot;14:00:00&quot;&gt; &lt;Content&gt;This is a blog post&lt;/Content&gt; &lt;Comments&gt; &lt;Comment&gt;This is a comment&lt;/Comment&gt; &lt;Comment&gt;This is another comment&lt;/Comment&gt; &lt;/Comments&gt; &lt;Draft&gt; &lt;Content&gt;This is a draft blog post&lt;/Content&gt; &lt;/Draft&gt; &lt;/Blog&gt; &quot;&quot;&quot; Let's define our schemas like so: defmodule DraftPost do import DataSchema , only : [ data_schema : 1 ] @data_accessor XpathAccessor data_schema ( [ field : { :content , &quot;./Content/text()&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end defmodule Comment do import DataSchema , only : [ data_schema : 1 ] @data_accessor XpathAccessor data_schema ( [ field : { :text , &quot;./text()&quot; , &amp; { :ok , to_string ( &amp;1 ) } } ] ) end defmodule BlogPost do import DataSchema , only : [ data_schema : 1 ] @data_accessor XpathAccessor @datetime_fields [ field : { :date , &quot;/Blog/@date&quot; , &amp; Date . from_iso8601 / 1 } , field : { :time , &quot;/Blog/@time&quot; , &amp; Time . from_iso8601 / 1 } , ] data_schema ( [ field : { :content , &quot;/Blog/Content/text()&quot; , &amp; { :ok , to_string ( &amp;1 ) } } , has_many : { :comments , &quot;//Comment&quot; , Comment } , has_one : { :draft , &quot;/Blog/Draft&quot; , DraftPost } , aggregate : { :post_datetime , @datetime_fields , &amp; NaiveDateTime . new ( &amp;1 . date , &amp;1 . time ) } , ] ) end And now we can transform: source_data = &quot;&quot;&quot; &lt;Blog date=&quot;2021-11-11&quot; time=&quot;14:00:00&quot;&gt; &lt;Content&gt;This is a blog post&lt;/Content&gt; &lt;Comments&gt; &lt;Comment&gt;This is a comment&lt;/Comment&gt; &lt;Comment&gt;This is another comment&lt;/Comment&gt; &lt;/Comments&gt; &lt;Draft&gt; &lt;Content&gt;This is a draft blog post&lt;/Content&gt; &lt;/Draft&gt; &lt;/Blog&gt; &quot;&quot;&quot; DataSchema . to_struct ( source_data , BlogPost ) # This will output: % BlogPost { comments : [ % Comment { text : &quot;This is a comment&quot; } , % Comment { text : &quot;This is another comment&quot; } ] , content : &quot;This is a blog post&quot; , draft : % DraftPost { content : &quot;This is a draft blog post&quot; } , post_datetime : ~N[2021-11-11 14:00:00] }","ref":"xml-schemas.html","title":"XML Schemas","type":"extras"}]